@@ -16,13 +16,14 @@ import ctypes
 import re
 import sys
 import time
-import LibAppArmor
-from apparmor.common import AppArmorException, AppArmorBug, hasher, open_file_read, split_name, DebugLogger
 
-# setup module translations
+import LibAppArmor
+from apparmor.common import AppArmorBug, AppArmorException, DebugLogger, hasher, open_file_read, split_name
 from apparmor.translations import init_translation
+
 _ = init_translation()
 
+
 class ReadLog:
 
     # used to pre-filter log lines so that we hand over only relevant lines to LibAppArmor parsing
@@ -32,15 +33,15 @@ class ReadLog:
         self.filename = filename
         self.profile_dir = profile_dir
         self.active_profiles = active_profiles
-        self.hashlog = { 'PERMITTING': {}, 'REJECTING': {}, 'AUDIT': {} }  # structure inside {}: {'profilename': init_hashlog(aamode, profilename), 'profilename2': init_hashlog(...), ...}
-        self.debug_logger = DebugLogger('ReadLog')
+        self.hashlog = {'PERMITTING': {}, 'REJECTING': {}}  # structure inside {}: {'profilename': init_hashlog(aamode, profilename), 'profilename2': init_hashlog(...), ...}
+        self.debug_logger = DebugLogger(type(self).__name__)
         self.LOG = None
         self.logmark = ''
         self.seenmark = None
         self.next_log_entry = None
 
     def init_hashlog(self, aamode, profile):
-        ''' initialize self.hashlog[aamode][profile] for all rule types'''
+        """initialize self.hashlog[aamode][profile] for all rule types"""
 
         if profile in self.hashlog[aamode].keys():
             return  # already initialized, don't overwrite existing data
@@ -56,6 +57,10 @@ class ReadLog:
             'path':         hasher(),
             'ptrace':       hasher(),
             'signal':       hasher(),
+            'userns':       hasher(),
+            'mqueue':       hasher(),
+            'io_uring':     hasher(),
+            'mount':        hasher(),
         }
 
     def prefetch_next_log_entry(self):
@@ -78,10 +83,7 @@ class ReadLog:
     def parse_event(self, msg):
         """Parse the event from log into key value pairs"""
         msg = msg.strip()
-        self.debug_logger.info('parse_event: %s' % msg)
-        if sys.version_info < (3, 0):
-            # parse_record fails with u'foo' style strings hence typecasting to string
-            msg = str(msg)
+        self.debug_logger.info('parse_event: %s', msg)
         event = LibAppArmor.parse_record(msg)
         ev = dict()
         ev['resource'] = event.info
@@ -104,6 +106,7 @@ class ReadLog:
         ev['family'] = event.net_family
         ev['protocol'] = event.net_protocol
         ev['sock_type'] = event.net_sock_type
+        ev['class'] = event._class
 
         if event.ouid != ctypes.c_ulong(-1).value:  # ULONG_MAX
             ev['fsuid'] = event.fsuid
@@ -114,6 +117,13 @@ class ReadLog:
             ev['peer'] = event.peer
         elif ev['operation'] and ev['operation'] == 'ptrace':
             ev['peer'] = event.peer
+        elif ev['operation'] and ev['operation'] == 'mount':
+            ev['flags'] = event.flags
+            ev['fs_type'] = event.fs_type
+            ev['src_name'] = event.src_name
+        elif ev['operation'] and (ev['operation'] == 'umount'):
+            ev['flags'] = event.flags
+            ev['fs_type'] = event.fs_type
         elif ev['operation'] and ev['operation'].startswith('dbus_'):
             ev['peer_profile'] = event.peer_profile
             ev['bus'] = event.dbus_bus
@@ -121,6 +131,9 @@ class ReadLog:
             ev['interface'] = event.dbus_interface
             ev['member'] = event.dbus_member
 
+        elif ev['operation'] and ev['operation'].startswith('uring_'):
+            ev['peer_profile'] = event.peer_profile
+
         LibAppArmor.free_record(event)
 
         if not ev['time']:
@@ -157,12 +170,12 @@ class ReadLog:
         if aamode == 'UNKNOWN':
             raise AppArmorBug('aamode is UNKNOWN - %s' % e['type'])  # should never happen
 
-        if aamode in ['AUDIT', 'STATUS', 'ERROR']:
-            return None
+        if aamode in ('AUDIT', 'STATUS', 'ERROR'):
+            return
 
         # Skip if AUDIT event was issued due to a change_hat in unconfined mode
         if not e.get('profile', False):
-            return None
+            return
 
         full_profile = e['profile']  # full, nested profile name
         self.init_hashlog(aamode, full_profile)
@@ -174,7 +187,7 @@ class ReadLog:
         profile, hat = split_name(e['profile'])
 
         if profile != 'null-complain-profile' and not self.profile_exists(profile):
-            return None
+            return
         if e['operation'] == 'exec':
             if not e['name']:
                 raise AppArmorException('exec without executed binary')
@@ -183,7 +196,34 @@ class ReadLog:
                 e['name2'] = ''  # exec events in enforce mode don't have target=...
 
             self.hashlog[aamode][full_profile]['exec'][e['name']][e['name2']] = True
-            return None
+            return
+
+        elif e['class'] and e['class'] == 'namespace':
+            if e['denied_mask'].startswith('userns_'):
+                self.hashlog[aamode][full_profile]['userns'][ e['denied_mask'][7:] ] = True  # [7:] removes the 'userns_' prefix
+            return
+
+        elif e['class'] and e['class'].endswith('mqueue'):
+            mqueue_type = e['class'].partition('_')[0]
+            self.hashlog[aamode][full_profile]['mqueue'][e['denied_mask']][mqueue_type][e['name']] = True
+            return
+
+        elif e['class'] and e['class'] == 'io_uring':
+            self.hashlog[aamode][full_profile]['io_uring'][e['denied_mask']][e['peer_profile']] = True
+            return
+
+        elif e['class'] and e['class'] == 'mount' or e['operation'] == 'mount':
+            if e['flags'] != None:
+                e['flags'] = ('=', e['flags'])
+            if e['fs_type'] != None:
+                e['fs_type'] = ('=', e['fs_type'])
+
+
+            if e['operation'] == 'mount':
+                self.hashlog[aamode][full_profile]['mount'][e['operation']][e['flags']][e['fs_type']][e['name']][e['src_name']] = True
+            else:  # Umount
+                self.hashlog[aamode][full_profile]['mount'][e['operation']][e['flags']][e['fs_type']][e['name']][None] = True
+            return
 
         elif self.op_type(e) == 'file':
             # Map c (create) and d (delete) to w (logging is more detailed than the profile language)
@@ -208,54 +248,57 @@ class ReadLog:
                 # in current log style, owner permissions are indicated by a match of fsuid and ouid
                 owner = True
 
+            if 'x' in dmask and dmask != 'x':
+                dmask = dmask.replace('x', '')  # if dmask contains x and another mode, drop x here - we should see a separate exec event
+
             for perm in dmask:
                 if perm in 'mrwalk':  # intentionally not allowing 'x' here
                     self.hashlog[aamode][full_profile]['path'][e['name']][owner][perm] = True
                 else:
                     raise AppArmorException(_('Log contains unknown mode %s') % dmask)
 
-            return None
+            return
 
         elif e['operation'] == 'capable':
             self.hashlog[aamode][full_profile]['capability'][e['name']] = True
-            return None
+            return
 
         elif self.op_type(e) == 'net':
             self.hashlog[aamode][full_profile]['network'][e['family']][e['sock_type']][e['protocol']] = True
-            return None
+            return
 
         elif e['operation'] == 'change_hat':
             if e['error_code'] == 1 and e['info'] == 'unconfined can not change_hat':
-                return None
+                return
 
             self.hashlog[aamode][full_profile]['change_hat'][e['name2']] = True
-            return None
+            return
 
         elif e['operation'] == 'change_profile':
             self.hashlog[aamode][full_profile]['change_profile'][e['name2']] = True
-            return None
+            return
 
         elif e['operation'] == 'ptrace':
             if not e['peer']:
                 self.debug_logger.debug('ignored garbage ptrace event with empty peer')
-                return None
+                return
             if not e['denied_mask']:
                 self.debug_logger.debug('ignored garbage ptrace event with empty denied_mask')
-                return None
+                return
 
             self.hashlog[aamode][full_profile]['ptrace'][e['peer']][e['denied_mask']] = True
-            return None
+            return
 
         elif e['operation'] == 'signal':
-            self.hashlog[aamode][full_profile]['signal'][e['peer']][e['denied_mask']][e['signal']]= True
-            return None
+            self.hashlog[aamode][full_profile]['signal'][e['peer']][e['denied_mask']][e['signal']] = True
+            return
 
-        elif e['operation'].startswith('dbus_'):
+        elif e['operation'] and e['operation'].startswith('dbus_'):
             self.hashlog[aamode][full_profile]['dbus'][e['denied_mask']][e['bus']][e['path']][e['name']][e['interface']][e['member']][e['peer_profile']] = True
-            return None
+            return
 
         else:
-            self.debug_logger.debug('UNHANDLED: %s' % e)
+            self.debug_logger.debug('UNHANDLED: %s', e)
 
     def read_log(self, logmark):
         self.logmark = logmark
@@ -266,32 +309,30 @@ class ReadLog:
             self.LOG = open_file_read(self.filename)
         except IOError:
             raise AppArmorException('Can not read AppArmor logfile: ' + self.filename)
-        line = True
-        while line:
-            line = self.get_next_log_entry()
-            if not line:
-                break
-            line = line.strip()
-            self.debug_logger.debug('read_log: %s' % line)
-            if self.logmark in line:
-                seenmark = True
+        with self.LOG:
+            line = True
+            while line:
+                line = self.get_next_log_entry()
+                if not line:
+                    break
+                line = line.strip()
+                self.debug_logger.debug('read_log: %s', line)
+                if self.logmark in line:
+                    seenmark = True
 
-            self.debug_logger.debug('read_log: seenmark = %s' % seenmark)
-            if not seenmark:
-                continue
+                self.debug_logger.debug('read_log: seenmark = %s', seenmark)
+                if not seenmark:
+                    continue
 
-            event = self.parse_event(line)
-            if event:
-                try:
-                    self.parse_event_for_tree(event)
+                event = self.parse_event(line)
+                if event:
+                    try:
+                        self.parse_event_for_tree(event)
+                    except AppArmorException as e:
+                        ex_msg = ('%(msg)s\n\nThis error was caused by the log line:\n%(logline)s'
+                                  % {'msg': e.value, 'logline': line})
+                        raise AppArmorBug(ex_msg) from None
 
-                except AppArmorException as e:
-                    ex_msg = ('%(msg)s\n\nThis error was caused by the log line:\n%(logline)s' %
-                            {'msg': e.value, 'logline': line})
-                    # when py3 only: Drop the original AppArmorException by passing None as the parent exception
-                    raise AppArmorBug(ex_msg)  # py3-only: from None
-
-        self.LOG.close()
         self.logmark = ''
 
         return self.hashlog
@@ -300,43 +341,45 @@ class ReadLog:
     # (used by op_type() which checks some event details to decide)
     OP_TYPE_FILE_OR_NET = {
         # Note: op_type() also uses some startswith() checks which are not listed here!
-       'create',
-       'post_create',
-       'bind',
-       'connect',
-       'listen',
-       'accept',
-       'sendmsg',
-       'recvmsg',
-       'getsockname',
-       'getpeername',
-       'getsockopt',
-       'setsockopt',
-       'socket_create',
-       'sock_shutdown',
-       'open',
-       'truncate',
-       'mkdir',
-       'mknod',
-       'chmod',
-       'chown',
-       'rename_src',
-       'rename_dest',
-       'unlink',
-       'rmdir',
-       'symlink',
-       'symlink_create',
-       'link',
-       'sysctl',
-       'getattr',
-       'setattr',
-       'xattr',
+        'create',
+        'post_create',
+        'bind',
+        'connect',
+        'listen',
+        'accept',
+        'sendmsg',
+        'recvmsg',
+        'getsockname',
+        'getpeername',
+        'getsockopt',
+        'setsockopt',
+        'socket_create',
+        'sock_shutdown',
+        'open',
+        'truncate',
+        'mkdir',
+        'mknod',
+        'chmod',
+        'chown',
+        'rename_src',
+        'rename_dest',
+        'unlink',
+        'rmdir',
+        'symlink',
+        'symlink_create',
+        'link',
+        'sysctl',
+        'getattr',
+        'setattr',
+        'xattr',
     }
 
     def op_type(self, event):
-        """Returns the operation type if known, unkown otherwise"""
+        """Returns the operation type if known, unknown otherwise"""
 
-        if ( event['operation'].startswith('file_') or event['operation'].startswith('inode_') or event['operation'] in self.OP_TYPE_FILE_OR_NET ):
+        if event['operation'] and (event['operation'].startswith('file_') or
+                                   event['operation'].startswith('inode_') or
+                                   event['operation'] in self.OP_TYPE_FILE_OR_NET):
             # file or network event?
             if event['family'] and event['protocol'] and event['sock_type']:
                 # 'unix' events also use keywords like 'connect', but protocol is 0 and should therefore be filtered out
