@@ -44,15 +44,16 @@ exception_not_raised = (
     'abi/bad_11.sd',
     'abi/bad_12.sd',
 
-    # invalid capabilities (like "foobar"), but syntactically correct
-    'capability/bad_1.sd',
-    'capability/bad_2.sd',
-    'capability/bad_3.sd',
-    'capability/bad_4.sd',
-
     # interesting[tm] profile name
     'change_hat/bad_parsing.sd',
 
+    'dbus/bad_regex_04.sd',
+    'dbus/bad_regex_05.sd',
+    'dbus/bad_regex_06.sd',
+    'file/bad_re_brace_1.sd',
+    'file/bad_re_brace_2.sd',
+    'file/bad_re_brace_3.sd',
+
     # The tools don't detect conflicting change_profile exec modes
     'change_profile/onx_conflict_unsafe1.sd',
     'change_profile/onx_conflict_unsafe2.sd',
@@ -76,49 +77,14 @@ exception_not_raised = (
     'file/bad_re_brace_1.sd',
     'file/bad_re_brace_2.sd',
     'file/bad_re_brace_3.sd',
-    'mount/bad_1.sd',
-    'mount/bad_2.sd',
-    'mount/bad_3.sd',
-    'mount/bad_4.sd',
-    'mount/bad_opt_10.sd',
-    'mount/bad_opt_11.sd',
-    'mount/bad_opt_12.sd',
-    'mount/bad_opt_13.sd',
-    'mount/bad_opt_14.sd',
-    'mount/bad_opt_15.sd',
-    'mount/bad_opt_16.sd',
-    'mount/bad_opt_17.sd',
-    'mount/bad_opt_18.sd',
-    'mount/bad_opt_19.sd',
-    'mount/bad_opt_1.sd',
-    'mount/bad_opt_20.sd',
-    'mount/bad_opt_21.sd',
-    'mount/bad_opt_22.sd',
-    'mount/bad_opt_23.sd',
-    'mount/bad_opt_24.sd',
-    'mount/bad_opt_2.sd',
-    'mount/bad_opt_3.sd',
-    'mount/bad_opt_4.sd',
-    'mount/bad_opt_5.sd',
-    'mount/bad_opt_6.sd',
-    'mount/bad_opt_7.sd',
-    'mount/bad_opt_8.sd',
-    'mount/bad_opt_9.sd',
-    'mount/bad_opt_25.sd',
-    'mount/bad_opt_26.sd',
-    'mount/bad_opt_27.sd',
-    'mount/bad_opt_28.sd',
+
+    # We do not check that options are compatible
     'mount/bad_opt_29.sd',
     'mount/bad_opt_30.sd',
     'mount/bad_opt_31.sd',
-    'mount/bad_opt_32.sd',
-    'mount/bad_opt_35.sd',
-    'mount/bad_opt_36.sd',
-    'mount/bad_opt_37.sd',
-    'mount/bad_opt_38.sd',
-    'mount/bad_opt_39.sd',
-    'mount/bad_opt_40.sd',
-    'mount/bad_opt_41.sd',
+    'mount/bad_1.sd',
+    'mount/bad_2.sd',
+
     'profile/flags/flags_bad10.sd',
     'profile/flags/flags_bad11.sd',
     'profile/flags/flags_bad12.sd',
@@ -126,7 +92,6 @@ exception_not_raised = (
     'profile/flags/flags_bad15.sd',
     'profile/flags/flags_bad18.sd',
     'profile/flags/flags_bad19.sd',
-    'profile/flags/flags_bad20.sd',
     'profile/flags/flags_bad2.sd',
     'profile/flags/flags_bad3.sd',
     'profile/flags/flags_bad4.sd',
@@ -137,7 +102,6 @@ exception_not_raised = (
     'profile/flags/flags_bad_debug_1.sd',
     'profile/flags/flags_bad_debug_2.sd',
     'profile/flags/flags_bad_debug_3.sd',
-    'profile/flags/flags_bad_debug_4.sd',
     # detection of conflicting flags not supported
     'profile/flags/flags_bad30.sd',
     'profile/flags/flags_bad31.sd',
@@ -156,9 +120,45 @@ exception_not_raised = (
     'profile/flags/flags_bad44.sd',
     'profile/flags/flags_bad45.sd',
     'profile/flags/flags_bad46.sd',
+    'profile/flags/flags_bad47.sd',
+    'profile/flags/flags_bad48.sd',
+    'profile/flags/flags_bad49.sd',
+    'profile/flags/flags_bad50.sd',
+    'profile/flags/flags_bad51.sd',
+    'profile/flags/flags_bad52.sd',
+    'profile/flags/flags_bad53.sd',
+    'profile/flags/flags_bad54.sd',
+    'profile/flags/flags_bad55.sd',
+    'profile/flags/flags_bad56.sd',
+    'profile/flags/flags_bad64.sd',
+    'profile/flags/flags_bad65.sd',
+    'profile/flags/flags_bad66.sd',
+    'profile/flags/flags_bad67.sd',
+    'profile/flags/flags_bad68.sd',
+    'profile/flags/flags_bad69.sd',
+    'profile/flags/flags_bad70.sd',
+    'profile/flags/flags_bad71.sd',
+    'profile/flags/flags_bad72.sd',
+    'profile/flags/flags_bad73.sd',
+    'profile/flags/flags_bad74.sd',
+    'profile/flags/flags_bad75.sd',
+    'profile/flags/flags_bad76.sd',
+    'profile/flags/flags_bad77.sd',
+    'profile/flags/flags_bad78.sd',
+    'profile/flags/flags_bad79.sd',
+    'profile/flags/flags_bad80.sd',
+    'profile/flags/flags_bad81.sd',
+    'profile/flags/flags_bad82.sd',
+    'profile/flags/flags_bad83.sd',
+    'profile/flags/flags_bad84.sd',
+    'profile/flags/flags_bad85.sd',
+    'profile/flags/flags_bad86.sd',
+    'profile/flags/flags_bad_disconnected_path1.sd',
+    'profile/flags/flags_bad_disconnected_path2.sd',
+    'profile/flags/flags_bad_disconnected_path3.sd',
+    'profile/flags/flags_bad_disconnected_path4.sd',
+    'profile/flags/flags_bad_disconnected_path5.sd',
     'profile/profile_ns_bad8.sd',  # 'profile :ns/t' without terminating ':'
-    'ptrace/bad_05.sd',  # actually contains a capability rule with invalid (ptrace-related) keyword
-    'ptrace/bad_06.sd',  # actually contains a capability rule with invalid (ptrace-related) keyword
     'ptrace/bad_10.sd',  # peer with invalid regex
     'signal/bad_21.sd',  # invalid regex
     'unix/bad_attr_1.sd',
@@ -313,6 +313,15 @@ unknown_line = (
     'bare_include_tests/ok_84.sd',
     'bare_include_tests/ok_85.sd',
     'bare_include_tests/ok_86.sd',
+
+    # According to spec mount should be in the form fstype=... options=... and NOT in the form options=... fstype=...
+    'mount/ok_opt_combo_3.sd',
+    'mount/ok_opt_combo_2.sd',
+    'mount/ok_opt_combo_1.sd',
+    'mount/ok_opt_combo_4.sd',
+
+    # Options should be comma separated
+    'mount/in_4.sd',  # also order option then fstype is invalid
 )
 
 # testcases with various unexpected failures
@@ -419,6 +428,95 @@ syntax_failure = (
     'vars/vars_dbus_8.sd',  # Path doesn't start with / or variable: {/@{TLDS}/foo,/com/@{DOMAINS}}
     'vars/vars_simple_assignment_12.sd',  # Redefining existing variable @{BAR} ('\' not handled)
     'bare_include_tests/ok_2.sd',  # two #include<...> in one line
+
+    # fine grained net
+    'network/network_ok_8.sd',
+    'network/network_ok_9.sd',
+    'network/network_ok_10.sd',
+    'network/network_ok_11.sd',
+    'network/network_ok_12.sd',
+    'network/network_ok_13.sd',
+    'network/network_ok_14.sd',
+    'network/network_ok_15.sd',
+    'network/network_ok_16.sd',
+    'network/network_ok_17.sd',
+    'network/network_ok_18.sd',
+    'network/network_ok_19.sd',
+    'network/network_ok_20.sd',
+    'network/network_ok_21.sd',
+    'network/network_ok_22.sd',
+    'network/network_ok_23.sd',
+    'network/network_ok_24.sd',
+    'network/network_ok_25.sd',
+    'network/network_ok_26.sd',
+    'network/network_ok_27.sd',
+    'network/network_ok_28.sd',
+    'network/network_ok_29.sd',
+    'network/network_ok_30.sd',
+    'network/network_ok_31.sd',
+    'network/network_ok_32.sd',
+    'network/network_ok_33.sd',
+    'network/network_ok_34.sd',
+    'network/network_ok_35.sd',
+    'network/network_ok_36.sd',
+    'network/network_ok_37.sd',
+    'network/network_ok_38.sd',
+    'network/network_ok_39.sd',
+    'network/network_ok_40.sd',
+    'network/network_ok_41.sd',
+    'network/network_ok_42.sd',
+    'network/network_ok_43.sd',
+    'network/network_ok_44.sd',
+    'network/perms/ok_accept_1.sd',
+    'network/perms/ok_accept_2.sd',
+    'network/perms/ok_attr_1.sd',
+    'network/perms/ok_attr_2.sd',
+    'network/perms/ok_attr_3.sd',
+    'network/perms/ok_attr_4.sd',
+    'network/perms/ok_attr_5.sd',
+    'network/perms/ok_attr_6.sd',
+    'network/perms/ok_attr_7.sd',
+    'network/perms/ok_attr_8.sd',
+    'network/perms/ok_bind_1.sd',
+    'network/perms/ok_bind_2.sd',
+    'network/perms/ok_connect_1.sd',
+    'network/perms/ok_connect_2.sd',
+    'network/perms/ok_create_1.sd',
+    'network/perms/ok_create_2.sd',
+    'network/perms/ok_create_3.sd',
+    'network/perms/ok_create_4.sd',
+    'network/perms/ok_listen_1.sd',
+    'network/perms/ok_listen_2.sd',
+    'network/perms/ok_listen_3.sd',
+    'network/perms/ok_msg_1.sd',
+    'network/perms/ok_msg_2.sd',
+    'network/perms/ok_msg_3.sd',
+    'network/perms/ok_msg_4.sd',
+    'network/perms/ok_msg_5.sd',
+    'network/perms/ok_msg_6.sd',
+    'network/perms/ok_msg_7.sd',
+    'network/perms/ok_msg_8.sd',
+    'network/perms/ok_msg_9.sd',
+    'network/perms/ok_msg_10.sd',
+    'network/perms/ok_msg_12.sd',
+    'network/perms/ok_msg_13.sd',
+    'network/perms/ok_msg_14.sd',
+    'network/perms/ok_msg_15.sd',
+    'network/perms/ok_msg_16.sd',
+    'network/perms/ok_msg_17.sd',
+    'network/perms/ok_msg_18.sd',
+    'network/perms/ok_msg_19.sd',
+    'network/perms/ok_msg_20.sd',
+    'network/perms/ok_opt_1.sd',
+    'network/perms/ok_opt_2.sd',
+    'network/perms/ok_opt_3.sd',
+    'network/perms/ok_opt_4.sd',
+    'network/perms/ok_opt_5.sd',
+    'network/perms/ok_opt_6.sd',
+    'network/perms/ok_opt_7.sd',
+    'network/perms/ok_shutdown_1.sd',
+    'network/perms/ok_shutdown_2.sd',
+    'network/perms/ok_shutdown_3.sd',
 )
 
 
@@ -474,7 +572,7 @@ def parse_test_profiles(file_with_path):
                 exresult = False
                 exresult_found = True
             else:
-                raise Exception('%s contains unknown EXRESULT %s' % (file_with_path, exresult))
+                raise Exception('{} contains unknown EXRESULT {}'.format(file_with_path, exresult))
 
         elif line.upper().startswith('#=DESCRIPTION '):
             description = line.split()[1]
@@ -486,25 +584,25 @@ def parse_test_profiles(file_with_path):
             disabled = True
 
     if not exresult_found:
-        raise Exception('%s does not contain EXRESULT' % file_with_path)
+        raise Exception(file_with_path + ' does not contain EXRESULT')
 
     if not description:
-        raise Exception('%s does not contain description' % file_with_path)
+        raise Exception(file_with_path + ' does not contain description')
 
     tools_wrong = False
     if relfile in exception_not_raised:
         if exresult:
-            raise Exception("%s listed in exception_not_raised, but has EXRESULT PASS" % file_with_path)
+            raise Exception(file_with_path + " listed in exception_not_raised, but has EXRESULT PASS")
         tools_wrong = 'EXCEPTION_NOT_RAISED'
     elif relfile.startswith(skip_startswith):
         return 1  # XXX *** SKIP *** those tests
     elif relfile in unknown_line:
         if not exresult:
-            raise Exception("%s listed in unknown_line, but has EXRESULT FAIL" % file_with_path)
+            raise Exception(file_with_path + " listed in unknown_line, but has EXRESULT FAIL")
         tools_wrong = 'UNKNOWN_LINE'
     elif relfile in syntax_failure:
         if not exresult:
-            raise Exception("%s listed in syntax_failure, but has EXRESULT FAIL" % file_with_path)
+            raise Exception(file_with_path + " listed in syntax_failure, but has EXRESULT FAIL")
         tools_wrong = 'SYNTAX_FAILURE'
 
     params = {
@@ -549,9 +647,9 @@ def find_and_setup_test_profiles(profile_dir):
                 skipped += parse_test_profiles(file_with_path)
 
     if skipped:
-        print('Skipping %s test profiles listed in skip_startswith.' % skipped)
+        print('Skipping {} test profiles listed in skip_startswith.'.format(skipped))
 
-    print('Running %s parser simple_tests...' % len(TestParseParserTests.tests))
+    print('Running {} parser simple_tests...'.format(len(TestParseParserTests.tests)))
 
 
 setup_aa(apparmor)
