@@ -18,7 +18,6 @@ from collections import namedtuple
 
 from apparmor.common import AppArmorBug, AppArmorException
 from apparmor.logparser import ReadLog
-from apparmor.rule import BaseRule
 from apparmor.rule.ptrace import PtraceRule, PtraceRuleset
 from apparmor.translations import init_translation
 from common_test import AATest, setup_all_loops
@@ -68,7 +67,7 @@ class PtraceTestParse(PtraceTest):
 
     def _run_test(self, rawrule, expected):
         self.assertTrue(PtraceRule.match(rawrule))
-        obj = PtraceRule.parse(rawrule)
+        obj = PtraceRule.create_instance(rawrule)
         self.assertEqual(rawrule.strip(), obj.raw_rule)
         self._compare_obj(obj, expected)
 
@@ -86,7 +85,7 @@ class PtraceTestParseInvalid(PtraceTest):
     def _run_test(self, rawrule, expected):
         self.assertTrue(PtraceRule.match(rawrule))  # the above invalid rules still match the main regex!
         with self.assertRaises(expected):
-            PtraceRule.parse(rawrule)
+            PtraceRule.create_instance(rawrule)
 
 
 class PtraceTestParseFromLog(PtraceTest):
@@ -181,7 +180,7 @@ class InvalidPtraceTest(AATest):
         obj = None
         self.assertFalse(PtraceRule.match(rawrule))
         with self.assertRaises(AppArmorException):
-            obj = PtraceRule.parse(rawrule)
+            obj = PtraceRule.create_instance(rawrule)
 
         self.assertIsNone(obj, 'PtraceRule handed back an object unexpectedly')
 
@@ -209,7 +208,7 @@ class InvalidPtraceTest(AATest):
 class WritePtraceTestAATest(AATest):
     def _run_test(self, rawrule, expected):
         self.assertTrue(PtraceRule.match(rawrule))
-        obj = PtraceRule.parse(rawrule)
+        obj = PtraceRule.create_instance(rawrule)
         clean = obj.get_clean()
         raw = obj.get_raw()
 
@@ -257,16 +256,16 @@ class WritePtraceTestAATest(AATest):
 
 class PtraceCoveredTest(AATest):
     def _run_test(self, param, expected):
-        obj = PtraceRule.parse(self.rule)
-        check_obj = PtraceRule.parse(param)
+        obj = PtraceRule.create_instance(self.rule)
+        check_obj = PtraceRule.create_instance(param)
 
         self.assertTrue(PtraceRule.match(param))
 
-        self.assertEqual(obj.is_equal(check_obj), expected[0], 'Mismatch in is_equal, expected %s' % expected[0])
-        self.assertEqual(obj.is_equal(check_obj, True), expected[1], 'Mismatch in is_equal/strict, expected %s' % expected[1])
+        self.assertEqual(obj.is_equal(check_obj), expected[0], 'Mismatch in is_equal, expected {}'.format(expected[0]))
+        self.assertEqual(obj.is_equal(check_obj, True), expected[1], 'Mismatch in is_equal/strict, expected {}'.format(expected[1]))
 
-        self.assertEqual(obj.is_covered(check_obj), expected[2], 'Mismatch in is_covered, expected %s' % expected[2])
-        self.assertEqual(obj.is_covered(check_obj, True, True), expected[3], 'Mismatch in is_covered/exact, expected %s' % expected[3])
+        self.assertEqual(obj.is_covered(check_obj), expected[2], 'Mismatch in is_covered, expected {}'.format(expected[2]))
+        self.assertEqual(obj.is_covered(check_obj, True, True), expected[3], 'Mismatch in is_covered/exact, expected {}'.format(expected[3]))
 
 
 class PtraceCoveredTest_01(PtraceCoveredTest):
@@ -437,7 +436,7 @@ class PtraceCoveredTest_08(PtraceCoveredTest):
 
 class PtraceCoveredTest_Invalid(AATest):
     def test_borked_obj_is_covered_1(self):
-        obj = PtraceRule.parse('ptrace read peer=/foo,')
+        obj = PtraceRule.create_instance('ptrace read peer=/foo,')
 
         testobj = PtraceRule('read', '/foo')
         testobj.access = ''
@@ -446,7 +445,7 @@ class PtraceCoveredTest_Invalid(AATest):
             obj.is_covered(testobj)
 
     def test_borked_obj_is_covered_2(self):
-        obj = PtraceRule.parse('ptrace read peer=/foo,')
+        obj = PtraceRule.create_instance('ptrace read peer=/foo,')
 
         testobj = PtraceRule('read', '/foo')
         testobj.peer = ''
@@ -455,25 +454,29 @@ class PtraceCoveredTest_Invalid(AATest):
             obj.is_covered(testobj)
 
     def test_invalid_is_covered(self):
-        obj = PtraceRule.parse('ptrace read,')
-
-        testobj = BaseRule()  # different type
+        raw_rule = 'ptrace read,'
+        class SomeOtherClass(PtraceRule):
+            pass
 
+        obj = PtraceRule.create_instance(raw_rule)
+        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
         with self.assertRaises(AppArmorBug):
             obj.is_covered(testobj)
 
     def test_invalid_is_equal_1(self):
-        obj = PtraceRule.parse('ptrace read,')
-
-        testobj = BaseRule()  # different type
+        raw_rule = 'ptrace read,'
+        class SomeOtherClass(PtraceRule):
+            pass
 
+        obj = PtraceRule.create_instance(raw_rule)
+        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
         with self.assertRaises(AppArmorBug):
             obj.is_equal(testobj)
 
     def test_invalid_is_equal_2(self):
-        obj = PtraceRule.parse('ptrace read,')
+        obj = PtraceRule.create_instance('ptrace read,')
 
-        testobj = PtraceRule.parse('ptrace read,')
+        testobj = PtraceRule.create_instance('ptrace read,')
         testobj.all_peers = False  # make testobj invalid (should trigger exception in _is_equal_aare())
 
         with self.assertRaises(AppArmorBug):
@@ -492,7 +495,7 @@ class PtraceLogprofHeaderTest(AATest):
     )
 
     def _run_test(self, params, expected):
-        obj = PtraceRule.parse(params)
+        obj = PtraceRule.create_instance(params)
         self.assertEqual(obj.logprof_header(), expected)
 
 
@@ -508,8 +511,7 @@ class PtraceRulesTest(AATest):
         self.assertEqual([], ruleset_2.get_clean(2))
 
         # test __repr__() for empty ruleset
-        as_string = '%s' % ruleset
-        self.assertEqual(as_string, '<PtraceRuleset (empty) />')
+        self.assertEqual(str(ruleset), '<PtraceRuleset (empty) />')
 
     def test_ruleset_1(self):
         ruleset = PtraceRuleset()
@@ -531,15 +533,14 @@ class PtraceRulesTest(AATest):
         ]
 
         for rule in rules:
-            ruleset.add(PtraceRule.parse(rule))
+            ruleset.add(PtraceRule.create_instance(rule))
 
         self.assertEqual(expected_raw, ruleset.get_raw())
         self.assertEqual(expected_clean, ruleset.get_clean())
 
         # test __repr__() for non-empty ruleset
-        as_string = '%s' % ruleset
         self.assertEqual(
-            as_string, '<PtraceRuleset>\n  ptrace peer=/foo,\n  ptrace read,\n</PtraceRuleset>')
+            str(ruleset), '<PtraceRuleset>\n  ptrace peer=/foo,\n  ptrace read,\n</PtraceRuleset>')
 
     def test_ruleset_2(self):
         ruleset = PtraceRuleset()
@@ -565,7 +566,7 @@ class PtraceRulesTest(AATest):
         ]
 
         for rule in rules:
-            ruleset.add(PtraceRule.parse(rule))
+            ruleset.add(PtraceRule.create_instance(rule))
 
         self.assertEqual(expected_raw, ruleset.get_raw(1))
         self.assertEqual(expected_clean, ruleset.get_clean(1))
