@@ -18,7 +18,6 @@ from collections import namedtuple
 
 from apparmor.common import AppArmorBug, AppArmorException
 # from apparmor.logparser import ReadLog
-from apparmor.rule import BaseRule
 from apparmor.rule.rlimit import RlimitRule, RlimitRuleset, split_unit
 from apparmor.translations import init_translation
 from common_test import AATest, setup_all_loops
@@ -71,7 +70,7 @@ class RlimitTestParse(RlimitTest):
 
     def _run_test(self, rawrule, expected):
         self.assertTrue(RlimitRule.match(rawrule))
-        obj = RlimitRule.parse(rawrule)
+        obj = RlimitRule.create_instance(rawrule)
         self.assertEqual(rawrule.strip(), obj.raw_rule)
         self._compare_obj(obj, expected)
 
@@ -95,7 +94,7 @@ class RlimitTestParseInvalid(RlimitTest):
     def _run_test(self, rawrule, expected):
         # self.assertFalse(RlimitRule.match(rawrule))  # the main regex isn't very strict
         with self.assertRaises(expected):
-            RlimitRule.parse(rawrule)
+            RlimitRule.create_instance(rawrule)
 
 
 class RlimitTestParseFromLog(RlimitTest):
@@ -181,7 +180,7 @@ class InvalidRlimitTest(AATest):
         obj = None
         self.assertFalse(RlimitRule.match(rawrule))
         with self.assertRaises(AppArmorException):
-            obj = RlimitRule.parse(rawrule)
+            obj = RlimitRule.create_instance(rawrule)
 
         self.assertIsNone(obj, 'RlimitRule handed back an object unexpectedly')
 
@@ -220,7 +219,7 @@ class WriteRlimitTest(AATest):
 
     def _run_test(self, rawrule, expected):
         self.assertTrue(RlimitRule.match(rawrule))
-        obj = RlimitRule.parse(rawrule)
+        obj = RlimitRule.create_instance(rawrule)
         clean = obj.get_clean()
         raw = obj.get_raw()
 
@@ -238,24 +237,24 @@ class WriteRlimitTest(AATest):
 
 class RlimitCoveredTest(AATest):
     def _run_test(self, param, expected):
-        obj = RlimitRule.parse(self.rule)
-        check_obj = RlimitRule.parse(param)
+        obj = RlimitRule.create_instance(self.rule)
+        check_obj = RlimitRule.create_instance(param)
 
         self.assertTrue(RlimitRule.match(param))
 
         self.assertEqual(
             obj.is_equal(check_obj), expected[0],
-            'Mismatch in is_equal, expected %s' % expected[0])
+            'Mismatch in is_equal, expected {}'.format(expected[0]))
         self.assertEqual(
             obj.is_equal(check_obj, True), expected[1],
-            'Mismatch in is_equal/strict, expected %s' % expected[1])
+            'Mismatch in is_equal/strict, expected {}'.format(expected[1]))
 
         self.assertEqual(
             obj.is_covered(check_obj), expected[2],
-            'Mismatch in is_covered, expected %s' % expected[2])
+            'Mismatch in is_covered, expected {}'.format(expected[2]))
         self.assertEqual(
             obj.is_covered(check_obj, True, True), expected[3],
-            'Mismatch in is_covered/exact, expected %s' % expected[3])
+            'Mismatch in is_covered/exact, expected {}'.format(expected[3]))
 
 
 class RlimitCoveredTest_01(RlimitCoveredTest):
@@ -336,7 +335,7 @@ class RlimitCoveredTest_05(RlimitCoveredTest):
 
 class RlimitCoveredTest_Invalid(AATest):
     def test_borked_obj_is_covered_1(self):
-        obj = RlimitRule.parse('set rlimit cpu <= 1024,')
+        obj = RlimitRule.create_instance('set rlimit cpu <= 1024,')
 
         testobj = RlimitRule('cpu', '1024')
         testobj.rlimit = ''
@@ -345,7 +344,7 @@ class RlimitCoveredTest_Invalid(AATest):
             obj.is_covered(testobj)
 
     def test_borked_obj_is_covered_2(self):
-        obj = RlimitRule.parse('set rlimit cpu <= 1024,')
+        obj = RlimitRule.create_instance('set rlimit cpu <= 1024,')
 
         testobj = RlimitRule('cpu', '1024')
         testobj.value = ''
@@ -354,18 +353,22 @@ class RlimitCoveredTest_Invalid(AATest):
             obj.is_covered(testobj)
 
     def test_invalid_is_covered(self):
-        obj = RlimitRule.parse('set rlimit cpu <= 1024,')
-
-        testobj = BaseRule()  # different type
+        raw_rule = 'set rlimit cpu <= 1024,'
+        class SomeOtherClass(RlimitRule):
+            pass
 
+        obj = RlimitRule.create_instance(raw_rule)
+        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
         with self.assertRaises(AppArmorBug):
             obj.is_covered(testobj)
 
     def test_invalid_is_equal(self):
-        obj = RlimitRule.parse('set rlimit cpu <= 1024,')
-
-        testobj = BaseRule()  # different type
+        raw_rule = 'set rlimit cpu <= 1024,'
+        class SomeOtherClass(RlimitRule):
+            pass
 
+        obj = RlimitRule.create_instance(raw_rule)
+        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
         with self.assertRaises(AppArmorBug):
             obj.is_equal(testobj)
 
@@ -379,7 +382,7 @@ class RlimitLogprofHeaderTest(AATest):
     )
 
     def _run_test(self, params, expected):
-        obj = RlimitRule.parse(params)
+        obj = RlimitRule.create_instance(params)
         self.assertEqual(obj.logprof_header(), expected)
 
 
@@ -414,7 +417,7 @@ class RlimitRulesTest(AATest):
         ]
 
         for rule in rules:
-            ruleset.add(RlimitRule.parse(rule))
+            ruleset.add(RlimitRule.create_instance(rule))
 
         self.assertEqual(expected_raw, ruleset.get_raw())
         self.assertEqual(expected_clean, ruleset.get_clean())
